local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local L = LibStub("AceLocale-3.0"):GetLocale("RW2-Plugin-Base")

local LSM = LibStub("LibSharedMedia-3.0")

local Boss = RW.BossPrototype
local sayProt = {}
local mt = {__index = sayProt}

local Party = RW.Party
local Utils = RW.Utils

local spell
local icon
function Boss:_Say(mType, text, optionName, default)
	if type(text) == "number" then 
		spell = text
		icon = select(3, GetSpellInfo(text))
		return self:_Say(mType, GetSpellInfo(text), optionName or text, default) 
	end
	
	optionName = type(optionName) == "number" and Utils.SpellName[optionName] or optionName
	
	-- If no option name is found by now we use the text as default
	if not optionName then optionName = type(text) == "table" and text[1] or text end
	
	self:AddToCategory(nil, optionName)
	
	default = Utils:BuildRoleDefaults(default)
	
	local defaults = self:AddDefaults(optionName, "SAY", mType ~= "CUSTOM" and mType or text[2], default)
	
	local base = self:GetBaseAlertObject(optionName)
	if not base then
		base = {			
			id = optionName,
			spell = spell,
			icon = icon,
			mod = self,
			scheduled = {},
		}
		tinsert(self.BaseAlertObjects, base)
	end
	
	local obj = setmetatable({
		mainType = "SAY",
		type = mType,	
		text = type(text) == "table" and text[1] or text,
		optText = (mType == "CUSTOM" and type(text) == "table") and text[2] or nil,
		base = base,
	}, mt)
	
	self.AlertObjects.SAY = self.AlertObjects.SAY or {}
	tinsert(self.AlertObjects.SAY, obj)
	
	spell = nil
	icon = nil
	
	return obj
end

function Boss:SayOnPlayer(...)
	return self:_Say("ON_PLAYER", ...)
end

function sayProt:Show(...)
	if not self.base.mod.DB[self.base.id].enabled then return end
	if not Utils:EvaluateRoleOption(self.base.mod.DB[self.base.id].SAY[self.optText or self.type].enabled) then return end
	
	local text
	if self.optText then	
		text = Utils:Format(self.text, ...)
	else
		text = Utils:Format(L["FORMAT_SAY_"..self.type], self.text or ..., ...)
	end
		
	SendChatMessage(text, "SAY")
end

-- Options
function sayProt:GetOption()
	local Options = RW:GetPlugin("Options")
	local boss = self.base.mod
	local AceGUI = LibStub("AceGUI-3.0")
	local UIF = LibStub("LibGUIFactory-1.0"):GetFactory("RW")

	local subGroup = UIF:InlineGroup(" ")
	
	local enabledDrop = UIF:MultiSelectDropdown(L.ENABLED, boss.DB[self.base.id].SAY[self.optText or self.type].enabled, nil, Utils.Constants.EnableList, 0.5)
	
	if self.type == "CUSTOM" then
		subGroup:SetTitle(self.optText)				
	else
		subGroup:SetTitle(L["SAY_"..self.type])
	end			
	
	subGroup:AddChild(enabledDrop)
	
	return subGroup
end 