
UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT = 14

GameTooltipStatusBar:SetHeight(7)
GameTooltipStatusBar:SetBackdrop({bgFile = 'Interface\\Buttons\\WHITE8x8'})
GameTooltipStatusBar:SetStatusBarTexture('Interface\\AddOns\\SharedMedia\\statusbar\\fer6.tga')

local gtt = GameTooltip;
local UnitExists = UnitExists;
local preFadeTime = 0;
local fadeTime = 0.2;
local updateFreq = 10;
local u = {};
local CLASS_COLORS = RAID_CLASS_COLORS;

-- Setup Gradient Tip
local function SetupGradientTip(tip)
	local g = tip.ttGradient;
	if (not g) then
		g = tip:CreateTexture();
		g:SetTexture(1,1,1,1);
		tip.ttGradient = g;
	end
	
	u.isPlayer = UnitIsPlayer(u.token);
	if (u.isPlayer) then
		local _, classEng = UnitClass(u.token);
		local color = CLASS_COLORS[classEng] or CLASS_COLORS["PRIEST"];
		g:SetGradientAlpha("VERTICAL",0,0,0,0,color.r,color.g,color.b,0.45);
	else
		g:SetGradientAlpha("VERTICAL",0,0,0,0,0.8,0.8,0.8,0.5);
	end
	g:SetPoint("TOPLEFT",5.5,5 * -1);
	g:SetPoint("BOTTOMRIGHT",tip,"TOPRIGHT",5.5 * -1,-36);
	g:Show();
end

-- Great Many functions :P
local function GameTooltip_GetUnit(self)
    if (GetMouseFocus() and not GetMouseFocus():GetAttribute('unit') and GetMouseFocus() ~= WorldFrame) then
        return select(2, self:GetUnit())
	elseif (GetMouseFocus() and GetMouseFocus():GetAttribute('unit')) then
		return GetMouseFocus():GetAttribute('unit')
    else
        return select(2, self:GetUnit())  
	end
end

local function GameTooltip_UnitCreatureType(unit)
    local creaturetype = UnitCreatureType(unit)
    if (creaturetype) then
        return creaturetype
    else
        return ''
    end
end

local function GameTooltip_UnitClassification(unit)
    local class = UnitClassification(unit)
    if (class == 'worldboss') then
        return '|cffFF0000'..BOSS..'|r '
    elseif (class == 'rareelite') then
        return '|cffFF66CCRare|r |cffFFFF00'..ELITE..'|r '
    elseif (class == 'rare') then 
        return '|cffFF66CCRare|r '
    elseif (class == 'elite') then
        return '|cffFFFF00'..ELITE..'|r '
    else
        return ''
    end
end

local function GameTooltip_UnitLevel(unit)
    local diff = GetQuestDifficultyColor(UnitLevel(unit))
    if (UnitLevel(unit) == -1) then
        return '|cffff0000??|r '
    elseif (UnitLevel(unit) == 0) then
        return '? '
    else
        return format('|cff%02x%02x%02x%s|r ', diff.r*255, diff.g*255, diff.b*255, UnitLevel(unit))    
    end
end

local function GameTooltip_UnitClass(unit)
    local color = RAID_CLASS_COLORS[select(2, UnitClass(unit))]
    if (color) then
        return format(' |cff%02x%02x%02x%s|r', color.r*255, color.g*255, color.b*255, UnitClass(unit))
    end
end

local function GameTooltip_UnitType(unit) 
    if (UnitIsPlayer(unit)) then
        return GameTooltip_UnitLevel(unit)..UnitRace(unit)..GameTooltip_UnitClass(unit)
    else
        return GameTooltip_UnitLevel(unit)..GameTooltip_UnitClassification(unit)..GameTooltip_UnitCreatureType(unit)
    end
end

GameTooltip.Icon = GameTooltip:CreateTexture('$parentRaidIcon', 'OVERLAY')
GameTooltip.Icon:SetPoint('TOPLEFT', GameTooltip, 10, -10)
GameTooltip.Icon:SetWidth(16)
GameTooltip.Icon:SetHeight(16)

local gtt_lastUpdate = 0;
local gtt_anchorType;

local function ApplyTooltipStyle(self)
    local bgsize, bsize
	
    if (self == ConsolidatedBuffsTooltip) then
        bgsize = 6
        bsize = 12
    else
        bgsize = 6
        bsize = 12
    end

	self:SetBackdrop({
		-- bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		tile = true,
		tileSize = 8,
		insets = { left = 5, right = 5, top = 5, bottom = 5 }
		});
	
	-- had to put this in here aswell for itemcompare tooltips ...
	self:SetBackdropColor(0, 0, 0, 0.8);
end

GameTooltip:HookScript('OnShow', function(tip)
	tip:SetBackdropColor(0, 0, 0, 0.8);
	SetupGradientTip(tip);
end)

hooksecurefunc("GameTooltip_ShowCompareItem", function(self)  
	if (not self) then
		self = GameTooltip
	end
    
    local shoppingTooltip1, shoppingTooltip2, shoppingTooltip3 = unpack(self.shoppingTooltips)
    
    if (not shoppingTooltip1.hasBorder) then
        ApplyTooltipStyle(shoppingTooltip1)
        shoppingTooltip1.hasBorder = true
    end
    
    if (not shoppingTooltip2.hasBorder) then
        ApplyTooltipStyle(shoppingTooltip2)
        shoppingTooltip2.hasBorder = true
    end
    
    if (not shoppingTooltip3.hasBorder) then
        ApplyTooltipStyle(shoppingTooltip3)
        shoppingTooltip3.hasBorder = true
    end
end)

for _, tooltip in pairs({
    GameTooltip,
    ItemRefTooltip,
    WorldMapTooltip,

    ShoppingTooltip1,
    ShoppingTooltip2,
    ShoppingTooltip3,   

    DropDownList1MenuBackdrop,
    DropDownList2MenuBackdrop,

    ConsolidatedBuffsTooltip,
    -- FrameStackTooltip,
    -- PaperDollStatTooltip,
    
    -- ChatMenu,
	-- EmoteMenu,
	-- LanguageMenu,
	-- DropDownList1,
}) do
    ApplyTooltipStyle(tooltip)
end

-- function to short-display HP value on StatusBar
local function ShortValue(value)
	if value >= 1e7 then
		return ('%.1fm'):format(value / 1e6):gsub('%.?0+([km])$', '%1')
	elseif value >= 1e6 then
		return ('%.2fm'):format(value / 1e6):gsub('%.?0+([km])$', '%1')
	elseif value >= 1e5 then
		return ('%.0fk'):format(value / 1e3)
	elseif value >= 1e3 then
		return ('%.1fk'):format(value / 1e3):gsub('%.?0+([km])$', '%1')
	else
		return value
	end
end

local gttFadeOut = gtt.FadeOut;
gtt.FadeOut = function(self,...)
	if (not u.token) then
		self.fadeOut = 2; -- Don't allow the OnUpdate handler to run the fadeout/update code
		gttFadeOut(self,...);
	elseif (preFadeTime == 0 and fadeTime == 0) then
		self:Hide();
	else
		self.fadeOut = 1;
		gtt_lastUpdate = 0;
	end
end

GameTooltip:HookScript("OnUpdate", function(self, elapsed, ...)
	-- This ensures that mouse anchored world frame tips have the proper color, their internal function seems to set them to a dark blue color
	if (self:GetAnchorType() == "ANCHOR_CURSOR") then
		self:SetBackdropColor(0,0,0,0.8);
		self:SetBackdropBorderColor(0,0,0,0);
		return;
	-- Anchor GTT to Mouse
	elseif (gtt_anchorType == "mouse") and (self.default) then
		AnchorFrameToMouse(self);
	end

	-- Fadeout / Update Tip if Showing a Unit
	-- Do not allow (fadeOut == 2), as that is only for non overridden fadeouts, but we still need to keep resizing the tip
	if (u.token) and (self.fadeOut ~= 2) then
		gtt_lastUpdate = (gtt_lastUpdate + elapsed);
		if (self.fadeOut) then
			self:Show(); -- Overrides self:FadeOut()
			if (gtt_lastUpdate > fadeTime + preFadeTime) then
				self.fadeOut = nil;
				self:Hide();
			elseif (gtt_lastUpdate > preFadeTime) then
				self:SetAlpha(1 - (gtt_lastUpdate - preFadeTime) / fadeTime);
			end
		-- This is only really useful for worldframe unit tips, when u.token == "mouseover", they do not call the GTT:FadeOut()
		elseif (not UnitExists(u.token)) then
			self:FadeOut();
		elseif (updateFreq > 0) and (gtt_lastUpdate > updateFreq) then
			gtt_lastUpdate = 0;
			-- self.Show();
		end
	end
end)

GameTooltip:HookScript('OnTooltipSetUnit', function(self, ...)
	local _, unit = self:GetUnit();
	-- Concated unit tokens such as "targettarget" cannot be returned as the unit by GTT:GetUnit() and it will return as "mouseover", but the "mouseover" unit is still invalid at this point for those unitframes!
	-- To overcome this problem, we look if the mouse is over a unitframe, and if that unitframe has a unit attribute set?
	if (not unit) then
		local mFocus = GetMouseFocus();
		unit = mFocus and mFocus:GetAttribute("unit");
	end
	-- A mage's mirror images sometimes doesn't return a unit, this would fix it
	if (not unit) and (UnitExists("mouseover")) then
		unit = "mouseover";
	end
	-- Sometimes when you move your mouse quicky over units in the worldframe, we can get here without a unit
	if (not unit) then
		self:Hide();
		return;
	end
	-- A "mouseover" unit is better to have as we can then safely say the tip should no longer show when it becomes invalid. Harder to say with a "party2" unit.
	-- This also helps fix the problem that "mouseover" units aren't valid for group members out of range, a bug that has been in WoW since 3.0.2 I think.
	if (UnitIsUnit(unit,"mouseover")) then
		unit = "mouseover";
	end
	-- We're done, apply appearance
	u.token = unit;
	self.fadeOut = nil; -- Az: Sometimes this wasn't getting reset, the fact a cleanup isn't performed at this point, now that it was moved to "OnTooltipCleared" is very bad, so this is a fix

	if (UnitExists(unit) and UnitName(unit) ~= UNKNOWN) then
        local name, realm = UnitName(unit)
        if UnitPVPName(unit) then name = UnitPVPName(unit) end
        
        local unitTargetName = UnitName(unit .. 'target');
		local unitTargetClassColor = RAID_CLASS_COLORS[select(2,UnitClass(unit .. 'target'))] or { r = 1, g = 0, b = 1 };
		local unitTargetReactionColor = { r = select(1, UnitSelectionColor(unit .. 'target')), g = select(2, UnitSelectionColor(unit .. 'target')), b = select(3, UnitSelectionColor(unit .. 'target')) };

		if (UnitIsPlayer(unit)) then
			local unitClassColor = RAID_CLASS_COLORS[select(2,UnitClass(unit))] or { r = 1, g = 0, b = 1 };
			GameTooltipTextLeft1:SetText(format('|cff%02x%02x%02x%s|r', unitClassColor.r*255, unitClassColor.g*255, unitClassColor.b*255, name),1,1,1)
		else
			GameTooltipTextLeft1:SetText(name)
		end
        
        if (GetGuildInfo(unit)) then
            if (GetGuildInfo(unit) == GetGuildInfo('player') and IsInGuild('player')) then
               GameTooltipTextLeft2:SetText('|cffFFFFCC'.. '<' ..GameTooltipTextLeft2:GetText().. '>' ..'|r')
            end
        end

        for i = 2, GameTooltip:NumLines() do
            if (_G['GameTooltipTextLeft'..i]:GetText():find('^'..TOOLTIP_UNIT_LEVEL:gsub('%%s', '.+'))) then
                _G['GameTooltipTextLeft'..i]:SetText(GameTooltip_UnitType(unit))
            end
        end
        
        if ( UnitExists(unit .. 'target') ) then
			if ( UnitName('player') == unitTargetName ) then
				self:AddLine('|cff00aaff' .. string.upper(YOU) .. '|r', 1, 1, 1)
			else
				if UnitIsPlayer(unit .. 'target') then
					self:AddLine(format('|cff%02x%02x%02x%s|r', unitTargetClassColor.r*255, unitTargetClassColor.g*255, unitTargetClassColor.b*255, unitTargetName), 1, 1, 1)
				else
					self:AddLine(format('|cff%02x%02x%02x%s|r', unitTargetReactionColor.r*255, unitTargetReactionColor.g*255, unitTargetReactionColor.b*255, unitTargetName), 1, 1, 1)
				end
			end
		end

		for i = 3, GameTooltip:NumLines() do
			if (_G['GameTooltipTextLeft'..i]:GetText():find(PVP_ENABLED)) then
				_G['GameTooltipTextLeft'..i]:SetText(nil)
                -- if (UnitIsPVPFreeForAll(unit)) then
                    -- GameTooltipTextLeft1:SetText('|cffFF0000# |r'..GameTooltipTextLeft1:GetText())
                -- elseif (UnitIsPVP(unit)) then
                    -- GameTooltipTextLeft1:SetText('|cff00FF00# |r'..GameTooltipTextLeft1:GetText())
                -- end
			end
		end
        
        if (GetRaidTargetIndex(unit) and not UnitIsDead(unit)) then
           GameTooltipTextLeft1:SetText('   '..GameTooltipTextLeft1:GetText())
            self.Icon:SetTexture('Interface\\TargetingFrame\\UI-RaidTargetingIcon_'..GetRaidTargetIndex(unit))
        end

        if (UnitIsAFK(unit)) then 
            self:AppendText(' |cffffffff[|r|cff00ff00AFK|r|cffffffff]|r')   
        elseif (UnitIsDND(unit)) then
            self:AppendText(' |cffffffff[|r|cff00ff00DND|r|cffffffff]|r')
        end

        if (realm and realm ~= '') then
            --self:AppendText(' (*)')
            self:AppendText(' - '..realm)
        end
        
        if (UnitIsDead(unit) or UnitIsGhost(unit)) then
            GameTooltipStatusBar:SetBackdropColor(0.5, 0.5, 0.5, 0.35)
        else
            GameTooltipStatusBar:SetBackdropColor(0, 0, 0, 0.8)
        end

        self:AddLine(' ')
        GameTooltipStatusBar:ClearAllPoints()
        GameTooltipStatusBar:SetPoint('LEFT', self:GetName()..'TextLeft'..self:NumLines(), 1, -3)
        GameTooltipStatusBar:SetPoint('RIGHT', self, -10, 0)
		
		--update HP value on status bar
		GameTooltipStatusBar:SetScript("OnValueChanged", function(self, value)
			if not value then
				return
			end
			local min, max = self:GetMinMaxValues()
			
			if (value < min) or (value > max) then
				return
			end
			local _, unit = GameTooltip:GetUnit()
			
			-- fix target of target returning nil
			if (not unit) then
				local GMF = GetMouseFocus()
				unit = GMF and GMF:GetAttribute("unit")
			end
			
			GameTooltipStatusBar:SetStatusBarColor(0.3, 0.7, 0.3, 0.9);
			
			if not self.text then
				self.text = self:CreateFontString(nil, 'LOW')
				self.text:SetPoint('CENTER', GameTooltipStatusBar, 0, 0)
				self.text:SetFont('Fonts\\NEUROPOL.ttf', 10, 'OUTLINE')
				self.text:Show()
				if unit then
					min, max = UnitHealth(unit), UnitHealthMax(unit)
					local hp = ShortValue(min).." / "..ShortValue(max)
					if UnitIsGhost(unit) then
						self.text:SetText('Ghost')
					elseif min == 0 or UnitIsDead(unit) or UnitIsGhost(unit) then
						self.text:SetText('Dead')
					else
						self.text:SetText(hp)
					end
				end
			else
				if unit then
					min, max = UnitHealth(unit), UnitHealthMax(unit)
					self.text:Show()
					local hp = ShortValue(min).." / "..ShortValue(max)
					if UnitIsGhost(unit) then
						self.text:SetText('Ghost')
					elseif min == 0 or UnitIsDead(unit) or UnitIsGhost(unit) then
						self.text:SetText('Dead')
					else
						self.text:SetText(hp)
					end
				else
					self.text:Hide()
				end
			end
		end)
    end
end)

GameTooltip:HookScript('OnTooltipCleared', function(self)
    GameTooltipStatusBar:ClearAllPoints()
    GameTooltipStatusBar:SetPoint('TOPLEFT', self, 'BOTTOMLEFT', 2, -2)
    GameTooltipStatusBar:SetPoint('TOPRIGHT', self, 'BOTTOMRIGHT', -2, -2)
    wipe(u);
    self.Icon:SetTexture(nil)
	gtt_lastUpdate = 0;
	self.fadeOut = nil;
end)

hooksecurefunc('GameTooltip_SetDefaultAnchor', function(tooltip, parent)
	tooltip:ClearAllPoints()
	tooltip:SetOwner(parent, "ANCHOR_NONE")
	AnchorFrameToMouse(tooltip);
end)

-- Anchor any given frame to mouse position
function AnchorFrameToMouse(frame)
	local x, y = GetCursorPosition();
	local effScale = frame:GetEffectiveScale();
	frame:ClearAllPoints();
	frame:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",(x / effScale),(y / effScale));
end
