local addon, ns = ...
local cfg = ns.cfg
local lib = ns.lib
oUF.colors.smooth = {.7, .15, .15, .85, .8, .45, .15, .15, .15}
  
-----------------------------
-- STYLE FUNCTIONS
-----------------------------
local function CreatePlayerStyle(self, unit, isSingle)
	self.mystyle = "player"
	lib.init(self)
	self.scale = cfg.scale
	self:SetSize(cfg.playerw, cfg.playerh)
	lib.gen_hpbar(self)
	lib.gen_hpstrings(self)
	lib.gen_highlight(self)
	lib.gen_ppbar(self)
	lib.gen_RaidMark(self)
	if cfg.showPlayerAuras then
		BuffFrame:Hide()
		lib.createBuffs(self)
		lib.createDebuffs(self)
	end
	self.Health.frequentUpdates = true
	self.Health.Smooth = true
	self.Power.colorClass = true
	self.Power.frequentUpdates = true
	self.Power.Smooth = true
	self.Power.bg.multiplier = 0.5
	lib.gen_castbar(self)
	lib.gen_InfoIcons(self)
	lib.TotemBars(self)
	lib.Experience(self)
	lib.Reputation(self)
	lib.AltPowerBar(self)
	lib.ThreatBar(self)
	if cfg.showPortrait then lib.gen_portrait(self) end
	if cfg.showRunebar then lib.genRunes(self) end
	if cfg.showHolybar then lib.genHolyPower(self) end
	if cfg.showShardbar then lib.genShards(self) end
	if cfg.showEclipsebar then lib.addEclipseBar(self) end
end
local function CreateTargetStyle(self, unit, isSingle)
	self.mystyle = "target"
	lib.init(self)
	self.scale = cfg.scale
	self:SetSize(cfg.targetw,cfg.targeth)
	lib.gen_hpbar(self)
	lib.gen_hpstrings(self)
	lib.gen_highlight(self)
	lib.gen_ppbar(self)
	lib.gen_RaidMark(self)
	self.Health.frequentUpdates = false
	self.Health.Smooth = true
	self.Power.colorTapping = true
	self.Power.colorDisconnected = true
	self.Power.colorHappiness = false
	self.Power.colorClass = true
	self.Power.colorReaction = true
	self.Power.Smooth = true
	self.Power.colorClass = true
	self.Power.bg.multiplier = 0.5
	lib.gen_castbar(self)
	lib.gen_mirrorcb(self)
	lib.addQuestIcon(self)
	lib.createAuras(self)
	lib.genCPoints(self)
	if cfg.showPortrait then lib.gen_portrait(self) end
end
local function CreateFocusStyle(self, unit, isSingle)
	
		self.mystyle = "focus"
		lib.init(self)
		-- Size and Scale
		self.scale = cfg.scale
		self:SetSize(cfg.focusw, cfg.focush)
		
		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		
		lib.gen_ppbar(self)
		self.Power.colorTapping = true
		self.Power.colorDisconnected = true
		self.Power.colorHappiness = false
		self.Power.colorClass = true
		self.Power.Smooth = true
		self.Power.colorReaction = true
		self.Power.colorHealth = true
		self.Power.bg.multiplier = 0.5
		
		lib.gen_RaidMark(self)

		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3
		
		lib.gen_castbar(self)
		lib.createAuras(self)
	
end
local function CreateToTStyle(self, unit, isSingle)

		self.mystyle = "tot"
		lib.init(self)
		-- Size and Scale
		self.scale = cfg.scale
		self:SetSize(cfg.totw, cfg.toth)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		-- lib.createAuras(self)
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3
	

end	
local function CreateFocusTargetStyle(self, unit, isSingle)
		
		self.mystyle = "focustarget"
		lib.init(self)
		-- Size and Scale
		self.scale = cfg.scale
		self:SetSize(cfg.ftargetw, cfg.ftargeth)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3	
end
local function CreatePetStyle(self, unit, isSingle)
		local _, playerClass = UnitClass("player")
		
		self.mystyle = "pet"
		lib.init(self)
		-- Size and Scale
		self.scale = cfg.scale
		self:SetSize(cfg.petw,cfg.peth)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3

		-- Hunter Pet Hapiness
		if PlayerClass == "HUNTER" then
			self.Power.colorReaction = false
			self.Power.colorClass = false
			self.Power.colorHappiness = true
		end
end
local function CreateBossStyle(self, unit, isSingle)
		-- unit specifics
		self.mystyle = "boss"
		
		-- Size and Scale
		self:SetSize(cfg.bossw, cfg.bossh)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		lib.createAuraWatch(self)
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3
end
local function CreateMTStyle(self)
		-- unit specifics
		self.mystyle = "oUF_MT"
		
		-- Size and Scale
		self:SetSize(120, 27)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3
end	
local function CreateArenaStyle(self)
		-- unit specifics
		self.mystyle = "arena"
		
		-- Size and Scale
		self:SetSize(cfg.arenaw, cfg.arenah)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		--self.Health.bg.multiplier = 0.3
end
local function CreatePartyStyle(self)
	self.mystyle = "party"
		lib.init(self)
		-- Size and Scale
		self.scale = cfg.scale
		self:SetSize(200,25)

		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_ppbar(self)
		lib.gen_RaidMark(self)
		lib.ReadyCheck(self)
		
		--lib.gen_portrait(self)
		--style specific stuff
		self.Health.frequentUpdates = false
		self.Health.Smooth = true
		----self.Health.bg.multiplier = 0.3
		self.Power.colorTapping = true
		self.Power.colorDisconnected = true
		self.Power.colorHappiness = false
		self.Power.colorClass = true
		self.Power.colorReaction = true
		self.Power.Smooth = true
		self.Power.colorClass = true
		self.Power.bg.multiplier = 0.5
		lib.createDebuffs(self)
		lib.gen_InfoIcons(self)
end
local function CreateRaidStyle(self)
				
		self.mystyle = "raid"
		
		-- Size and Scale
		self.scale = cfg.scale
		
		self.Range = {
			insideAlpha = 1,
			outsideAlpha = .3,
		}
		
		if cfg.HealFrames then
		lib.gen_ppbar(self)
		self.Power.colorTapping = true
		self.Power.colorDisconnected = true
		self.Power.colorHappiness = false
		self.Power.colorClass = true
		self.Power.colorReaction = true
		self.Power.Smooth = true
		self.Power.colorClass = true
		self.Power.bg.multiplier = 0.5
		end
		-- Generate Bars
		lib.gen_hpbar(self)
		lib.gen_hpstrings(self)
		lib.gen_highlight(self)
		lib.gen_RaidMark(self)
		lib.ReadyCheck(self)

		--style specific stuff
		self.Health.frequentUpdates = true
		self.Health.Smooth = true
		self.Health.colorSmooth = true
		

		lib.CreateTargetBorder(self)
		lib.CreateThreatBorder(self)
		lib.debuffHighlight(self)
		lib.raidDebuffs(self)
		lib.gen_InfoIcons(self)
		lib.createAuraWatch(self)
		lib.createIndicators(self, unit)	
		lib.healcomm(self, unit)
		
		self.Health.PostUpdate = lib.PostUpdateRaidFrame
		self:RegisterEvent('PLAYER_TARGET_CHANGED', lib.ChangedTarget)
		self:RegisterEvent('RAID_ROSTER_UPDATE', lib.ChangedTarget)
		self:RegisterEvent("UNIT_THREAT_LIST_UPDATE", lib.UpdateThreat)
		self:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE", lib.UpdateThreat)
	
end
 
-----------------------------
-- SPAWN UNITS
-----------------------------

oUF:RegisterStyle("Player", CreatePlayerStyle)
oUF:RegisterStyle("Target", CreateTargetStyle)
oUF:RegisterStyle("ToT", CreateToTStyle)
oUF:RegisterStyle("Focus", CreateFocusStyle)
oUF:RegisterStyle("FocusTarget", CreateFocusTargetStyle)
oUF:RegisterStyle("Pet", CreatePetStyle)
oUF:RegisterStyle("Boss", CreateBossStyle)
oUF:RegisterStyle("Raid", CreateRaidStyle)
oUF:RegisterStyle("oUF_MT", CreateMTStyle)
oUF:RegisterStyle("Party", CreatePartyStyle)
oUF:RegisterStyle("Arena", CreateArenaStyle)

oUF:Factory(function(self)
	if cfg.HealFrames then
	self:SetActiveStyle("Player")
	local player = self:Spawn("player", "oUF_Player")
	player:SetPoint("TOPRIGHT", UIParent, "BOTTOM", cfg.playerx, cfg.playery+100)
	self:SetActiveStyle("Target")
	local target = self:Spawn("Target", "oUF_Target")
	target:SetPoint("TOPLEFT", UIParent, "BOTTOM", cfg.targetx, cfg.targety+100)
	if cfg.showtot then
		self:SetActiveStyle("ToT")
		local targettarget = self:Spawn("targettarget", "oUF_tot")
		targettarget:SetPoint("BOTTOM", UIParent, "BOTTOM", cfg.totx, cfg.toty+100) 
	end
	if cfg.showpet then
		self:SetActiveStyle("Pet")
		local pet = self:Spawn("pet", "oUF_pet")
		pet:SetPoint("BOTTOM", UIParent, "BOTTOM", cfg.petx, cfg.pety+100) 
	end
	if cfg.showfocus then 
		self:SetActiveStyle("Focus")
		local focus = self:Spawn("focus", "oUF_focus")
		focus:SetPoint("RIGHT", cfg.focusx, cfg.focusy) 
	end
	if cfg.showfocustarget then 
		self:SetActiveStyle("FocusTarget")
		local focustarget = self:Spawn("focustarget", "oUF_focustarget")
		focustarget:SetPoint("RIGHT", oUF_focus, "LEFT", -10, 0)
	end
	
	else
	
	self:SetActiveStyle("Player")
	local player = self:Spawn("player", "oUF_Player")
	player:SetPoint("TOPRIGHT", UIParent, "BOTTOM", cfg.playerx, cfg.playery)
	self:SetActiveStyle("Target")
	local target = self:Spawn("Target", "oUF_Target")
	target:SetPoint("TOPLEFT", UIParent, "BOTTOM", cfg.targetx, cfg.targety)
	oUF:SetActiveStyle("Party")
	local party = {}
	for i = 1, 5 do
		party[i] = oUF:Spawn("party"..i, "oUF_Party"..i)
		if i == 1 then
			party[i]:SetPoint("LEFT", UIParent, "LEFT", cfg.partyx, cfg.partyy)
		else
			party[i]:SetPoint("BOTTOMLEFT", party[i-1], "BOTTOMLEFT", 0, 40)
		end
	end
	if cfg.showtot then
		self:SetActiveStyle("ToT")
		local targettarget = self:Spawn("targettarget", "oUF_tot")
		targettarget:SetPoint("BOTTOM", UIParent, "BOTTOM", cfg.totx, cfg.toty) 
	end
	if cfg.showpet then
		self:SetActiveStyle("Pet")
		local pet = self:Spawn("pet", "oUF_pet")
		pet:SetPoint("BOTTOM", UIParent, "BOTTOM", cfg.petx, cfg.pety) 
	end
	if cfg.showfocus then 
		self:SetActiveStyle("Focus")
		local focus = self:Spawn("focus", "oUF_focus")
		focus:SetPoint("RIGHT", cfg.focusx, cfg.focusy)
	end
	if cfg.showfocustarget then 
		self:SetActiveStyle("FocusTarget")
		local focustarget = self:Spawn("focustarget", "oUF_focustarget")
		focustarget:SetPoint("RIGHT", oUF_focus, "LEFT", -10, 0)
	end
	end
	if cfg.MTFrames then
		oUF:SetActiveStyle("oUF_MT")
		local tank = oUF:SpawnHeader('oUF_MT', nil, 'raid',
			'oUF-initialConfigFunction', ([[
				self:SetWidth(%d)
				self:SetHeight(%d)
			]]):format(cfg.MTw, cfg.MTh),
			'showRaid', true,
			'groupFilter', 'MAINTANK',
			'yOffset', 3,
			'point' , 'TOP',
			'template', 'oUF_MainTank'
			)
		tank:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", cfg.MTx, cfg.MTy)
    end
  	if cfg.showBossFrames then
		self:SetActiveStyle("Boss")
		-- local boss = {}
			-- for i = 1, MAX_BOSS_FRAMES do
				-- boss[i] = self:Spawn("boss"..i, "oUF_Boss"..i)
				-- if i == 1 then
					-- boss[i]:SetPoint("RIGHT", UIParent, "RIGHT", cfg.bossx, cfg.bossy)
				-- else
					-- boss[i]:SetPoint("BOTTOMRIGHT", boss[i-1], "BOTTOMRIGHT", 0, 30)
			-- end
		-- end
	end
	if cfg.ArenaFrames then
	oUF:SetActiveStyle("Arena")
		local arena = {}
		for i = 1, 5 do
			arena[i] = oUF:Spawn("arena"..i, "oUF_Arena"..i)
			if i == 1 then
				arena[i]:SetPoint("RIGHT", UIParent, "RIGHT", cfg.arenax, cfg.arenay)
			else
				arena[i]:SetPoint("BOTTOMRIGHT", arena[i-1], "BOTTOMRIGHT", 0, 30)
			end
		end
	end

	-- Kill(CompactPartyFrame)
	if cfg.HealFrames then
	if cfg.ShowRaid then
		CompactRaidFrameContainer:Hide() 
		CompactRaidFrameManager:SetAlpha(0)
		self:SetActiveStyle("Raid")
		local maxGroups = 5
		if cfg.RaidShowAllGroups then maxGroups = 8 end
		local raid = oUF:SpawnHeader("oUF_Raid", nil, "solo,party,raid", 
		"showRaid", true,  
		"showPlayer", true,
		"showSolo", cfg.RaidShowSolo,
		"showParty", cfg.ShowParty,
		"xoffset", 7,
		"yOffset", 15,
		"groupFilter", "1,2,3,4,5,6,7,8",
		"groupBy", "GROUP",
		"groupingOrder", "1,2,3,4,5,6,7,8",
		"sortMethod", "INDEX",
		"maxColumns", maxGroups,
		"unitsPerColumn", 5,
		"columnSpacing", 7,
		"point", "BOTTOM",
		"columnAnchorPoint", "RIGHT",
		"oUF-initialConfigFunction", ([[
		self:SetWidth(%d)
		self:SetHeight(%d)
		]]):format(cfg.raidw, cfg.raidh))
		raid:SetPoint("BOTTOMRIGHT", UIParent, cfg.raidx, cfg.raidy)
	end
	else
	if cfg.ShowRaid then
		CompactRaidFrameContainer:Hide() 
		CompactRaidFrameManager:SetAlpha(0)
		self:SetActiveStyle("Raid")
		local maxGroups = 5
		if cfg.RaidShowAllGroups then maxGroups = 8 end
		local raid = oUF:SpawnHeader("oUF_Raid", nil, "solo,party,raid", 
		"showRaid", true,  
		"showPlayer", true,
		"showSolo", cfg.RaidShowSolo,
		"showParty", cfg.ShowParty,
		"xoffset", 7,
		"yOffset", 7,
		"groupFilter", "1,2,3,4,5,6,7,8",
		"groupBy", "GROUP",
		"groupingOrder", "1,2,3,4,5,6,7,8",
		"sortMethod", "INDEX",
		"maxColumns", maxGroups,
		"unitsPerColumn", 5,
		"columnSpacing", 7,
		"point", "BOTTOM",
		"columnAnchorPoint", "RIGHT",
		"oUF-initialConfigFunction", ([[
		self:SetWidth(%d)
		self:SetHeight(%d)
		]]):format(cfg.raidw, cfg.raidh))
		raid:SetPoint("BOTTOMRIGHT", UIParent, cfg.raidx, cfg.raidy)
	end
	end
end)
