local addon, ns = ...
local cfg = ns.cfg

local SVal = function(val)
	if val then
		if (val >= 1e6) then
			return ("%.1fm"):format(val / 1e6)
		elseif (val >= 1e3) then
			return ("%.1fk"):format(val / 1e3)
		else
			return ("%d"):format(val)
		end
	end
end

local function hex(r, g, b)
	if r then
		if (type(r) == 'table') then
			if(r.r) then r, g, b = r.r, r.g, r.b else r, g, b = unpack(r) end
		end
		return ('|cff%02x%02x%02x'):format(r * 255, g * 255, b * 255)
	end
end

oUF.Tags['drk:hp']  = function(u) 
  if UnitIsDead(u) or UnitIsGhost(u) or not UnitIsConnected(u) then
    return oUF.Tags['drk:DDG'](u)
  else
	local per = oUF.Tags['perhp'](u).."%" or 0
    local min, max = UnitHealth(u), UnitHealthMax(u)
    if u == "player" or u == "target" then
      if min~=max then 
        return SVal(min).." | "..per
      else
        return SVal(max).." | "..per
      end
    else
      return per
    end
  end
end
oUF.TagEvents['drk:hp'] = 'UNIT_HEALTH'

oUF.Tags['drk:power']  = function(u) 
	local min, max = UnitPower(u), UnitPowerMax(u)
	if min~=max then 
		return SVal(min)
	else
		return SVal(max)
	end
end
oUF.TagEvents['drk:power'] = 'UNIT_POWER UNIT_MAXPOWER'


oUF.Tags['drk:raidhp']  = function(u) 
  if UnitIsDead(u) or UnitIsGhost(u) or not UnitIsConnected(u) then
    return oUF.Tags['drk:DDG'](u)
  else
	local per = oUF.Tags['perhp'](u).."%" or 0
    return per
  end
end
oUF.TagEvents['drk:raidhp'] = 'UNIT_HEALTH'

oUF.Tags['drk:color'] = function(u, r)
	local _, class = UnitClass(u)
	local reaction = UnitReaction(u, "player")
	
	if UnitIsDead(u) or UnitIsGhost(u) or not UnitIsConnected(u) then
		return "|cffA0A0A0"
	elseif (UnitIsTapped(u) and not UnitIsTappedByPlayer(u)) then
		return hex(oUF.colors.tapped)
	elseif (u == "pet") then
		return hex(oUF.colors.reaction[reaction])
	elseif (UnitIsPlayer(u)) then
		return hex(oUF.colors.class[class])
	elseif reaction then
		return hex(oUF.colors.reaction[reaction])
	else
		return hex(1, 1, 1)
	end
end
oUF.TagEvents['drk:color'] = 'UNIT_REACTION UNIT_HEALTH UNIT_HAPPINESS'

oUF.Tags["drk:afkdnd"] = function(unit) 
	
	return UnitIsAFK(unit) and "|cffCFCFCF <afk>|r" or UnitIsDND(unit) and "|cffCFCFCF <dnd>|r" or ""
end
oUF.TagEvents["drk:afkdnd"] = "PLAYER_FLAGS_CHANGED"

oUF.Tags['drk:DDG'] = function(u)
	if UnitIsDead(u) then
		return "|cffCFCFCF Dead|r"
	elseif UnitIsGhost(u) then
		return "|cffCFCFCF Ghost|r"
	elseif not UnitIsConnected(u) then
		return "|cffCFCFCF D/C|r"
	end
end
oUF.TagEvents['drk:DDG'] = 'UNIT_HEALTH'



-- ComboPoints
oUF.Tags["myComboPoints"] = function(unit)

	local cp, str		
	if(UnitExists'vehicle') then
		cp = GetComboPoints('vehicle', 'target')
	else
		cp = GetComboPoints('player', 'target')
	end

	if (cp == 1) then
		str = string.format("|cff69e80c%d|r",cp)
	elseif cp == 2 then
		str = string.format("|cffb2e80c%d|r",cp)
	elseif cp == 3 then
		str = string.format("|cffffd800%d|r",cp) 
	elseif cp == 4 then
		str = string.format("|cffffba00%d|r",cp) 
	elseif cp == 5 then
		str = string.format("|cfff10b0b%d|r",cp)
	end
	
	return str
end
oUF.TagEvents["myComboPoints"] = "UNIT_COMBO_POINTS PLAYER_TARGET_CHANGED"

-- Deadly Poison Tracker
oUF.Tags["myDeadlyPoison"] = function(unit)

	local Spell = "Deadly Poison" or GetSpellInfo(43233)
	local ct = hasUnitDebuff(unit, Spell)
	local cp = GetComboPoints('player', 'target')
	
	if cp > 0 then
		if (not ct) then
			str = ""
		elseif (ct == 1) then
			str = string.format("|cffc1e79f%d|r",ct)
		elseif ct == 2 then
			str = string.format("|cfface678%d|r",ct)
		elseif ct == 3 then
			str = string.format("|cff9de65c%d|r",ct) 
		elseif ct == 4 then
			str = string.format("|cff8be739%d|r",ct) 
		elseif ct == 5 then
			str = string.format("|cff90ff00%d|r",ct)
		end
	else
		str = ""
	end
	
	return str
end

oUF.TagEvents["myDeadlyPoison"] = "UNIT_COMBO_POINTS PLAYER_TARGET_CHANGED UNIT_AURA"

-- Level
oUF.Tags["drk:level"] = function(unit)
	
	local c = UnitClassification(unit)
	local l = UnitLevel(unit)
	local d = GetQuestDifficultyColor(l)
	
	local str = l
		
	if l <= 0 then l = "??" end
	
	if c == "worldboss" then
		str = string.format("|cff%02x%02x%02xBoss|r",250,20,0)
	elseif c == "eliterare" then
		str = string.format("|cff%02x%02x%02x%s|r|cff0080FFR|r+",d.r*255,d.g*255,d.b*255,l)
	elseif c == "elite" then
		str = string.format("|cff%02x%02x%02x%s|r+",d.r*255,d.g*255,d.b*255,l)
	elseif c == "rare" then
		str = string.format("|cff%02x%02x%02x%s|r|cff0080FFR|r",d.r*255,d.g*255,d.b*255,l)
	else
		if not UnitIsConnected(unit) then
			str = "??"
		else
			if UnitIsPlayer(unit) then
				str = string.format("|cff%02x%02x%02x%s",d.r*255,d.g*255,d.b*255,l)
			elseif UnitPlayerControlled(unit) then
				str = string.format("|cff%02x%02x%02x%s",d.r*255,d.g*255,d.b*255,l)
			else
				str = string.format("|cff%02x%02x%02x%s",d.r*255,d.g*255,d.b*255,l)
			end
		end		
	end
	
	return str
end

oUF.TagEvents["drk:level"] = "UNIT_LEVEL PLAYER_LEVEL_UP UNIT_CLASSIFICATION_CHANGED"

local _, class = UnitClass("player")
local spells
oUF.Indicators = {
	["TL"] = "",
	["TR"] = "",
	["BL"] = "",
	["BR"] = ""
}
if (class == "DRUID") then
	spells = {
		["Mark of the Wild"] = GetSpellInfo(1126)
	}
	oUF.Tags["MotW"] = function(unit)
		if (not UnitAura(unit, spells["Mark of the Wild"])) then
			return "|cffFF00FFM|r"
		end
	end
	oUF.TagEvents["MotW"] = "UNIT_AURA"
	oUF.Indicators["TR"] = "[MotW]"
	
elseif (class == "DEATHKNIGHT") then
	spells = {
		["Horn of Winter"] = GetSpellInfo(57330)
	}
	oUF.Tags["HoW"] = function(unit)
		if (UnitAura(unit, spells["Horn of Winter"])) then
			return "|cffffff10M|r"
		end
	end
	oUF.TagEvents["HoW"] = "UNIT_AURA"
	oUF.Indicators["TR"] = "[HoW]"
elseif (class == "HUNTER") then
	spells = {}
elseif (class == "MAGE") then
	spells = {
		["Focus Magic"] = GetSpellInfo(54646)
	}
	oUF.Tags["FM"] = function(unit)
		if (UnitAura(unit, spells["Focus Magic"])) then
			return "|cffffff00M|r"
		end
	end
	oUF.TagEvents["FM"] = "UNIT_AURA"
	
	oUF.Indicators["TR"] = "[FM]"
elseif (class == "PALADIN") then
	spells = {
		["Beacon of Light"] = GetSpellInfo(53563),
		["Blessing of Kings"] = GetSpellInfo(20217),
		["Blessing of Might"] = GetSpellInfo(19740)
	}
	
	oUF.Tags["BoL"] = function(unit)
		if (UnitAura(unit, spells["Beacon of Light"])) then
			return "|cffffff10M|r"
		end
	end
	oUF.TagEvents["BoL"] = "UNIT_AURA"
	
	oUF.Tags["sBoL"] = function(unit)
		local _, _, _, _, _, _, _, caster = UnitAura(unit, spells["Beacon of Light"])
		if (caster and caster == "player") then
			return "|cffff33ffM|r"
		end
	end
	oUF.TagEvents["sBoL"] = "UNIT_AURA"
	
	oUF.Tags["Blessing"] = function(unit)
		if (not (select(8, UnitAura(unit, spells["Blessing of Kings"])) == "player" or
			select(8, UnitAura(unit, spells["Blessing of Might"])) == "player")) then
			return "|cffffff00M|r"
		end
	end
	oUF.TagEvents["Blessing"] = "UNIT_AURA"
	
	oUF.Indicators["TR"] = "[sBoL][BoL][Blessing]"
	
elseif (class == "PRIEST") then
	spells = {
		["Power Word: Fortitude"] = GetSpellInfo(21562),
		["Shadow Protection"] = GetSpellInfo(27683)
	}
	oUF.Tags["PW:F"] = function(unit)
		if (not UnitAura(unit, spells["Power Word: Fortitude"])) then
			return "|cff00A1DEM|r"
		end
	end
	oUF.TagEvents["PW:F"] = "UNIT_AURA"
	oUF.Tags["SP"] = function(unit)
		if (not UnitAura(unit, spells["Shadow Protection"])) then
			return "|cff9900FFM|r"
		end
	end
	oUF.TagEvents["SP"] = "UNIT_AURA"
	oUF.Indicators["TR"] = "[SP][PW:F]"

elseif (class == "ROGUE") then
	spells = {}
elseif (class == "SHAMAN") then
	spells = {}
elseif (class == "WARLOCK") then
	spells = {}
elseif (class == "WARRIOR") then
	spells = {
		["Battle Shout"] = GetSpellInfo(6673),
		["Commanding Shout"] = GetSpellInfo(469),
		["Vigilance"] = GetSpellInfo(50720)
	}
	oUF.Tags["BS"] = function(unit)
		if (UnitAura(unit, spells["Battle Shout"])) then
			return "|cffff0000M|r"
		end
	end
	oUF.TagEvents["BS"] = "UNIT_AURA"
	
	oUF.Tags["CS"] = function(unit)
		if (UnitAura(unit, spells["Commanding Shout"])) then
			return "|cffffff00M|r"
		end
	end
	oUF.TagEvents["CS"] = "UNIT_AURA"
	oUF.Indicators["TR"] = "[BS][CS]"
end